COMMENT /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   (C) Copyright Intel Corp., 1996

   File: altxface.asm

   Universal Serial Bus 930 Test firmware

   Revision History
   ---------------------------------------------
    0.2     05-27-96          Abdul Rahman Ismail
    0.1     02-13-96          Joe DrAgony
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

INCLUDE "82930.inc"
INCLUDE "swap.inc"

INCLUDE "ONE_OH.INC"

;------------- External functions - process.c
EXTERN	_SetFeature:CODE
EXTERN	_ClearFeature:CODE
EXTERN  _GetStatus:CODE
EXTERN  _SetInterface:CODE

;------------- External functions - service.c
EXTERN	_ServiceEndPoints:CODE
EXTERN	_CheckEPRunning:CODE
EXTERN  _CheckEPOverflow:CODE

;------------- External variables - process.c

;------------- Global functions
PUBLIC	_FillEP1Fifo
PUBLIC	_FillEP2Fifo

;------------- Global variables.
XDEF	_EP1RxPtr, _EP1TxPtr, _EP1DataCount

XDEF	_EP2RxPtr, _EP2TxPtr, _EP2DataCount

XDEF	_EP3RxPtr, _EP3TxPtr, _EP3DataCount

XDEF    _gbEP1Running
XDEF    _gbEP2Running
XDEF	_gbMPSEndPoint1
XDEF	_gbMPSEndPoint2

; TO DO
; Change Packet Transfer to use Words not bytes.
FIFO_SIZE               equ     08h
MPS_EP1_AX0             equ     08h

CSEG  AT FF:0000H
        LJMP main

CSEG  AT FF:003BH       ;EP-0
        LJMP EP1_ISR

CSEG  AT FF:0043H       ;EP-1
        LJMP EP0_ISR

CSEG  AT FF:004BH       ;EP-2
        LJMP EP2_ISR

CSEG  AT FF:0053H       ;EP-3
        LJMP EP3_ISR

CSEG  AT FF:0100H
main:
        mov     P1, #00h
        mov     SP, #00h
        mov     SPH, #05h               ; Set the stack to start @ 00:0500h
        mov     IE0, #00h               ; Disable all interrupts
        mov     DPXL, #0ffh             ; Set DPXL to point at the FLASH device to access constants
        lCall   INIT_VARIABLES          ; Initialize the RAM space as required
        lCall   INIT_USB                ; Initialize the USB subsystem

        mov     R8, #MPS_EP1_AX0
        mov     00:_gbMPSEndPoint1, R8
        mov     00:_gbMPSEndPoint2, R8

        mov     R8, #00h
        mov     00:_gbEP1Running, R8
        mov     00:_gbEP2Running, R8

        lCall   _FillEP1Fifo
        lCall   _FillEP2Fifo

        mov     R8, #01h
        lCall   _FillEP1Fifo
        lCall   _FillEP2Fifo

        ljmp    ActiveLoop              ; Jump over the ISR stubs to the main loop.

;Interrupts
EP1_ISR:
         RETI

EP2_ISR:
         RETI

EP3_ISR:
         RETI

EP0_ISR:
        ;CLR IE_USB.0
        RETI

COMMENT *------------------------------------------------------------
Function name     : _FillEP1Fifo
Brief Description : Fills one of the TX fifos. R8 contains SOFL
Regs preserved    : No reg. is saved
--------------------------------------------------------------------*
SCOPE
_FillEP1Fifo:
        mov     R7, 00:_gbMPSEndPoint1

?FillFifo:
        mov     TXDAT1, R8
        djnz    R7, ?FillFifo

        mov     R7, 00:_gbMPSEndPoint1
        mov     TXCNT1, R7

?Return:
        Ret

COMMENT *------------------------------------------------------------
Function name     : _FillEP2Fifo
Brief Description : Fills one of the TX fifos. R8 contains SOFL
Regs preserved    : No reg. is saved
--------------------------------------------------------------------*
SCOPE
_FillEP2Fifo:
        mov     R7, 00:_gbMPSEndPoint2

?FillFifo:
        mov     TXDAT2, R8
        djnz    R7, ?FillFifo

        mov     R7, 00:_gbMPSEndPoint2
        mov     TXCNT2, R7

?Return:
        Ret

COMMENT *------------------------------------------------------------
Function name     : ActiveLoop
Brief Description : Monitors RX and TX on EP0. Also checks activity
		  : on recption of a SOF. If the endpoints are
		  : running then updates the fifos with the present
		  : SOF value.
Regs preserved    : No reg. is saved
--------------------------------------------------------------------*
SCOPE
ActiveLoop:
EP0_RX:
        jnb     RXD0, EP0_TX
        lCall   OUT_TOKEN1              ; 82930 has received a packet from HC

EP0_TX:
        jnb     TXD0, ?CheckSOFRecd
        lCall   IN_TOKEN                ; 82930 has sent a packet to HC

?CheckSOFRecd:
        jnb  	SOFH.6, ?CheckEPRunning

        mov  	R8, SOFL
        clr     SOFH.6
        lCall   _ServiceEndPoints

?CheckEPRunning:
        lCall   _CheckEPRunning

?CheckEPOVF:
        lCall   _CheckEPOverflow

        jmp     ActiveLoop

SCOPE
;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
; This is the block that services EP0 events.
;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
;####################### FAKE INTERUPT ROUTINE #######################
;
; SetupSeq    Meaning
;   0          No Setup command pending.  Any out received is treated as a
;              DeviceRequest. Later steppings will actually detect a SETUP
;              packet and set a status flag telling us SETUP was received.
;
;   1          DeviceRequest Received, Processing Get Request. An IN Will
;              transfer data.  An Out will end the transfer and set
;              SetupSeq to 0
;
;   2          DeviceRequest Received, Processing Set Request. An OUT Will
;              transfer data to the buffer. An IN  will end the transfer and
;              set SetupSeq to 0
;
;   3          Done bit SetRequest.  Ignore all data sent until an IN
;              Used for Set Address

; This block of code deals with Endpoint 0 getting a transmit event interrupt.

OUT_TOKEN1:
        anl     SBI, #EP0_RX_CLR     ; Clear the interrupt bit.

        ;Check sequence bit for the state that we are in.
        mov   WR28, #00        ; point to page 00
        mov   WR30, #LOW16(SetupSeq)
        mov   R11, @DR28
        cmp   R11, #0          ; Is this a setup packet??
        jne   ?l1              ; If not, then deal with the data phase.
        lcall SETUP_TOKEN      ; If yes, then parse the setup command.
        ljmp  ?lx              ; Return to caller.

        ; If this is not a setup command then it must be one of two things
        ; 0 or more data packets for a set command (Seq=2) or an
        ; status result if Seq=1

?l1:    cmp R11, #1       ; Is this the status phase of a "GET" command?
        jne ?l2           ; No, so check next
        SETB RXCON0.4    ;Update receive FIFO state
        SETB TXCON0.7    ;Flush the Transmit FIFOS
        CLR EPCON0.1     ;disable Tx Output on EP0. Respond to "IN"
                         ;with "NAK"
        mov WR30,#LOW16(SetupSeq)    ;Update the state machine-Expect a setup token
        mov r11,#0
        mov @DR28, R11
        sjmp ?lx

        ;jb RXSTAT0.6, ?l1   ; If not SETUP then must be a standard out

?l2:    cmp R11, #2      ; Are we processing a Control Write, i.e. Set Descr...
        jne ?l3
        lcall OUT_TOKEN  ; If we are continue to read in the data.
        SJMP ?lx

?l3:    cmp R11, #3      ;If 3, then we are done setting the address and
                         ;all outs are to be ignored until an IN terminates
                         ;the control transfer
        jne ?l4
        SETB RXCON0.4    ;Clear the receive FIFO
        sjmp ?lx

?l4:

?lx:    ret

SCOPE
;#####################################################################
;#################### IN TOKEN  ######################################
;#####################################################################
; SetupSeq    Meaning
;   0          No Setup command pending.  Any out received is a DeviceRequest
;
;   1          DeviceRequest Received, Processing Get Request. An IN Will
;              transfer data.  An Out will end the transfer and set
;              SetupSeq to 0
;
;   2          DeviceRequest Received, Processing Set Request. An OUT Will
;              transfer data to the buffer. An IN  will end the transfer and
;              set SetupSeq to 0
;
;   3          Done SetADDRESS Request.  Ignore all data sent until an IN
;              Used for Set Address
;
;
; For in tokens we can do 2 things.
;
; If SetupSeq = 2,3 then a IN will terminate the control transfer.


SCOPE
IN_TOKEN:
        anl     SBI, #EP0_TX_CLR        ; clear the interrupt bit
        mov     R11, 00:SetupSeq        ; read state variable
        cmp     R11, #02h               ; Should this be the end to a setup sequence?
        jne     InTokenCheckSA          ; If no,check for end of set address

?StatusPhaseDone:
        mov     R11, #00h
        mov     00:SetupSeq, R11        ; Set state var. to expect setup.
        jmp     ?Return

InTokenCheckSA:
        cmp     R11, #03h               ; Prev. Setup was SetAddress???
        jne     ?SendDataBack
        mov     R10, 00:new_addr
        mov     FADDR, R10
        mov     R11, #00h
        mov     00:SetupSeq, R11
        jmp     ?Return

?SendDataBack:
        cmp     R11, #01h
        jne     ?Return

        lCall    DoControlOutput

?Return:
        Ret

SCOPE
;***************** SETUP_TOKEN  *************************
;***** Gets and saves SETUP DATA ************************
;******************************************************

;Procedure that takes SETUP data from the FIFO, Puts it in memory
;and then updates writes it to the fifo.
SETUP_TOKEN:
        MOV WR28, #0           ; USE DR28 for our data pointer
        mov WR30, #LOW16(COMMAND_BUFFER)
        mov R8, RXCNT0          ; Use R8 for our count register
        mov @DR28, R8           ; Store the count in the buffer
        add WR30, #1           ; Increment mem pointer
        orl R8, R8              ; Set flags
        je  ?k2                ; Zero length SETUP, error out....

                               ; Read FIFO loop
?k1:    mov R11, RXDAT0        ; Read byte from FIFO into R11
        mov @DR28, R11         ; Store it in the buffer
        ADD WR30, #1            ; Increment the pointer
        sub R8, #1             ; Decrement the counter
        jne ?k1                ; Read until FIFO empty

        setb RXCON0.4          ; Set FFRC bit to update RX FIFO state
        sjmp PROCESS_SETUP

?k2:                         ;Should never get a zero length setup.
        setb RXCON0.4        ;Clear the fifo
        LJMP EXIT_SETUP      ;leave.
SCOPE

;***************** PROCESS_SETUP **********************
;***** PROCESS THE SETUP ******************************
;******************************************************
SCOPE
PROCESS_SETUP:

       ;Compare bmRequest Type
       ;If it's it 1xxxxxxb - Get Descriptor/Get Configuration
       ;If it's it 0xxxxxxb - Set Address/Configuration/Set Feature/Clear Feature.

        mov WR28, #0
        mov WR30, #LOW16(bmRequestType)
        mov R11, @DR28

        anl R11, #80h;   Get/Set ??
        jne DO_GET_COMMAND

       LJMP DO_SET_COMMAND

;===================================================
; Routines to process a get descriptor/configuration
;===================================================

DO_GET_COMMAND:                            ;Is this a get_config or a Get_Descrip.
    mov WR30, #LOW16(bRequest)
    mov R11, @DR28
    cmp R11, #GET_DESCRIPTOR
    jne CheckGetConfig
    ljmp GET_DESCR_Request             ;Go get Descriptor (Device & Config)

CheckGetConfig:
    cmp R11, #GET_CONFIGURATION
    jne CheckGetStatus
    ljmp GET_CONFIG_Request            ;Go get config request

; code for other types of bRequests
CheckGetStatus:
	cmp	R11, #GET_STATUS
	jne 	CheckGetInterface
	mov 	WR12, wIndex
	SWAP_BYTES R12,R13,R14
	push 	WR12				;wIndex
	mov 	R12, bmRequestType
	push 	R12				;bmRequestType

	lCall 	_GetStatus            		;Go get status
        sub	SPX, #03h
	; WR0 contains the result returned from GetStatus
	; need to swap the bytes to make it little endian for the bus
	SWAP_BYTES R0, R1, R2

	mov 	R2, #01h 
	mov	SetupSeq , R2			;Advance State Machine to next state

	lCall	SetStatusData
	lCall   DoControlOutput		;primes fifo

	ljmp 	EXIT_SETUP

CheckGetInterface:
    cmp R11, #GET_INTERFACE
    jne CheckSynchFrame
    ljmp GET_CONFIG_Request            ;Go get config request

CheckSynchFrame:
    cmp R11, #SYCH_FRAME
    jne GetCommandExit
    ljmp GET_CONFIG_Request            ;Go get config request

GetCommandExit:
    ljmp EXIT_SETUP

;===================================================
; Routines to process a set descriptor/configuration
;===================================================

DO_SET_COMMAND:
	mov	WR30, #LOW16(bRequest)
	mov	R11, @DR28

	cmp	R11, #SET_DESCRIPTOR ; Was this a Set_Descriptor?
        jne     ?CheckSetConfig
        jmp	DO_SET_DESCRIPTOR

?CheckSetConfig:
        cmp	R11, #SET_CONFIGURATION ; Was this a Set_Configuration?
        jne     ?CheckSetInterface
        jmp	DO_SET_CONFIGURATION

?CheckSetInterface:
        cmp	R11, #SET_INTERFACE         ; Was this a Set_Interface?
        jne     ?CheckClearFeature
        jmp	DO_SET_INTERFACE

?CheckClearFeature:
        cmp	R11, #CLEAR_FEATURE
        jne     ?CheckSetAddress
        jmp	DO_CLEAR_FEATURE

?CheckSetAddress:
	cmp     R11, #SET_ADDRESS   ; Was this a Set_Address?
        jne     ?CheckSetFeature
	jmp	DO_SET_ADDRESS

?CheckSetFeature:
	cmp	R11, #SET_FEATURE   ; Was this a Set_Feature?
        jne     ?CheckExit
	jmp	DO_SET_FEATURE
        
?CheckExit:
        ajmp DO_SET_COMMAND_EXIT

DO_SET_DESCRIPTOR:
;
       ajmp DO_SET_COMMAND_EXIT

DO_SET_CONFIGURATION:
;
       ajmp DO_SET_COMMAND_EXIT

DO_SET_INTERFACE:
	mov	WR12, wIndex
        SWAP_BYTES R12, R13, R14
        push	WR12

	mov	WR12, wValue
        SWAP_BYTES R12, R13, R14
        push	WR12

        lCall	_SetInterface
        sub	SPX, #04h

        jmp DO_SET_COMMAND_EXIT

DO_CLEAR_FEATURE:
	mov	WR12, wIndex
        SWAP_BYTES R12, R13, R14
        push	WR12

	mov	WR12, wValue
        SWAP_BYTES R12, R13, R14
        push	WR12

        mov	R12, bmRequestType
        push	R12
        LCall	_ClearFeature
        sub	SPX, #05h

	jmp DO_SET_COMMAND_EXIT

DO_SET_FEATURE:
	mov	WR12, wIndex
        SWAP_BYTES R12, R13, R14
        push	WR12

	mov	WR12, wValue
        SWAP_BYTES R12, R13, R14
        push	WR12

        mov	R12, bmRequestType
        push	R12
        LCall	_SetFeature
        sub	SPX, #05h

	jmp DO_SET_COMMAND_EXIT

DO_SET_ADDRESS:
       mov WR30, #LOW16(wValue)   ; Get the address from wValue and store it.
       mov WR8, @DR28            ; It doesn't actually get changed until the
       mov WR30, #LOW16(new_addr) ;status phase completes successfully.
        mov @DR28, R8
       mov WR30, #LOW16(SetupSeq)
       mov R10, #3                ; This was a Set Address command.
       mov @DR28, R10             ;Set state to look for status phase.
       ajmp DO_SET_COMMAND_EXIT

DO_SET_COMMAND_EXIT:

       mov TXCNT0, #0          ; Prime FIFO for response to status phase
       setb EPCON0.1          ; Set the Tx output enable bit.
       ajmp EXIT_SETUP


        ; ********************** GET DESCRIPTOR *******************
GET_DESCR_Request:

       mov WR30, #LOW16(wValue)
       mov WR10, @DR28
       cmp R11, #DEVICE_DESCR    ;Low Byte in Rll
       jne ?c1


       ; ********************** GET DESCRIPTOR,  DEVICE *******************

       mov WR30, #LOW16(SetupSeq)   ;Advance State Machine to next state
       mov R10, #1                  ;Set state to doing a "GET" command
       mov @DR28, R10               ;

       mov WR30, #LOW16(gDevDescrBuffer)
       LCALL BufferControlData
       LCALL DoControlOutput
       ljmp EXIT_SETUP


?c1:   cmp R11, #CONFIG_DESCR
       jne ?c2
       ; ********************** GET DESCRIPTOR,  CONFIGURATION ************

       mov WR30, #LOW16(SetupSeq)    ;Update state machine
       mov R10, #1
       mov @DR28, R10

       mov WR30, #LOW16(gConfigDescrBuffer);
       LCALL BufferControlData
       LCALL DoControlOutput

       ljmp EXIT_SETUP

?c2:
       ljmp EXIT_SETUP

GET_CONFIG_Request:



EXIT_SETUP:
       ret

SCOPE

;***************** OUT_TOKEN  *************************
;***** Gets and saves OUT DATA ************************
;******************************************************

;Program that takes data from the FIFO, Puts it in memory
;and then updates writes it to the fifo.
OUT_TOKEN:                     ; Get the data and place it in the out_buffers
        anl SBI, #EP0_RX_CLR        ; Clear the bit
        MOV WR24, #0           ; USE DR24
        mov WR26, #LOW16(OUT_BUFFERS)
        mov R8, RXCNT0         ; Read count of bytes rec'd into R8
        mov @DR24, R8          ; Store the count in the OUT_BUFFERS space.
        add WR26, #1           ; Increment mem pointer
        orl R8, R8             ; Set flags
        je  ?k2

?k1:    mov R7, RXDAT0         ; Read loop
        mov @DR24, R7
        ADD WR26, #1
        sub R8, #1
        jne ?k1

?k2:    setb RXCON0.4          ; Clear the fifo
                               ; We now have the data in the buffer.
                               ; Transmit it to the xmit fifo
        ret

SCOPE
;###################################################################
;################### BufferControlData ############################
;## Transfers the data pointed to by WR30 into the IN Buffer #######
;###################################################################

SCOPE
BufferControlData:
        mov     R6, #00h
	mov	R7, @WR30		; Move Count
	inc     WR30, #01h		; increment pointer to first byte of data
	mov	R9, #00h

	mov	in_rd_pntr, R9		;Set buffer pointer to zero
					;for new transfer

        mov     WR10, wLength           ; cmp no of bytes asked for with the
	SWAP_BYTES R10,R11,R12          ; no. of bytes that the descriptor
        cmp     WR6, WR10               ; contains. 
        jle     ?Continue

        mov     WR6, WR10               ; If the no. of bytes in the descriptor
                                        ; is greater than the no. of bytes asked for
                                        ; send back only the no. of bytes as per
                                        ; the Setup packet.

?Continue:
	mov	in_bytes_left, R7       ;Store Count
	mov	gwControlBufferStart, WR30

	Ret

; ---------------------------------------------------------
;
;
; ---------------------------------------------------------
SetStatusData:
	mov	R2, #00h
	mov	in_rd_pntr, R2       ;Set buffer pointer to zero

	mov	R2, #02h
	mov	in_bytes_left, R2    ;point to count variable

	mov	WR26, #LOW16(gOtherControlBuffer)
	mov	gwControlBufferStart, WR26

	mov	@WR26, WR0

	Ret

;###################################################################
;################# DoControlOutput ##################################
;###################################################################

SCOPE
DoControlOutput:
        clr     EPCON0.1                ; Set it up so that data is not Xmitted
                                        ; while were are filling up the fifo.
	mov	R10, in_bytes_left
	cmp	R10, #00h		;check for none
	jne	CntlDataAvail    	;if ther is data do normal flow
        mov	R8, #00h		;if none, do null packet
	ljmp	ControlArmTx

CntlDataAvail:
        mov	WR26, gwControlBufferStart      ;Get the address of the first char
        mov	R12, #00h                       ;set up a word value for add since
        mov	R13, in_rd_pntr                 ;this is just a byte value.

        add	WR26, WR12		;Add the read_pointer to the buffer location
        mov	R8, #00h	        ;initialize FIFO full counter
        push	R10                     ;save starting bytes avail count

?WriteToFifo:
        mov	R11, @WR26              ;Get the data
        mov	TXDAT0, R11             ;write it into the FIFO
        add	WR26, #01h              ;Increment buffer pointer
        add     R8,#01h                 ;Increment FIFO count
        sub     R10, #01h               ;Are we done with the buffer
        je	ControlTxUpd

        cmp     R8, #FIFO_SIZE		;FIFO FULL?
        jne	?WriteToFifo

ControlTxUpd:
        pop     R10                     ;restore starting count
        mov	R12, in_rd_pntr		;Get the offset into IN_BUFFER
        add	R12, R8		        ;add amount we consumed
        mov     in_rd_pntr, R12         ;store the new offset
        sub     R10, R8                 ;subtract amount we just consumed
        mov     in_bytes_left, R10      ;store new count

ControlArmTx:
        mov     TXCNT0, R8              ;Write count into TXCNT register
        setb    EPCON0.1                ;Enable data transmit

?Return:
        Ret

SCOPE
;*****************************************************
;***************** INIT_USB  *************************
;*****************************************************

INIT_USB:
        mov     TXCON0, #84h    ;/* Flush FIFO, sets Empty bit in TXFLG0, enable ATM. */
        mov     TXCON1, #8Ch    ;/* Flush FIFO, sets Empty bit in TXFLG1, enable ATM. ISOC */
        mov     TXCON2, #8Ch    ;/* Flush FIFO, sets Empty bit in TXFLG1, enable ATM. ISOC */

        mov     RXCON0, #84h    ;/* Flush FIFO, enable ATM */

        ;  Enable endpoint 0 as a control endpoint. We need to set the
        ;  Transmit Endpoint enable, Receive Endpoint enable, and the
        ;  Receive Input enable. We do not set the Transmit Output enable
        ;  until we have determined what type of response we need to send
        ;  back, i.e., NULL packet for status phase or data for a "GET"
        ;  command.
        mov     EPCON0, #2Dh;

        ; Enable transmits on endpoint 1.
        mov     EPCON1, #03h; ;/* Enable transmit on EP1 and EP2 */
        mov     EPCON2, #03h
        mov     EPCON3, #00h

        Ret


;************************************************************************
;************* DEVICE DESCRIPTOR ****************************************
;************************************************************************

;
; NOTE!!!!! '251 is a Big Endian machine. Words and DWords are stored with the
; LSB in the numerically higher address.
;

DEVICE_DESCRIPTOR:
        gDLength:                   db  012h  ;Used by packet transfer routines.
        gDevice_bLength:            db  012h
        gDevice_bDescriptorType:    db  DEVICE_DESCR
        gDevice_bcdUSB:             dw  0001h  ; Version 1.00 compliant
        gDevice_bDeviceClass:       db  0DCh
        gDevice_bDeviceSubClass:    db  00h
        gDevice_bDeviceProtocol:    db  00h
        gDevice_wMaxPacketSize0:    db  08h    ;8 byte max for EP0
        gDevice_widVendor:          dw  8680h  ;Intel Vendor ID (8086)
        gDevice_widProduct:         dw  9098h  ;product ID 9876h
        gDevice_bcdDevice:          dw  3001h  ;device version 1.30
        gDevice_iManufacturer:      db  0h     ;These three fields are supposed
        gDevice_iProduct:           db  0h     ;to contain the index of strings
        gDevice_iSerialNumber:      db  0h     ;describing device.
        gDevice_bNumConfigurations: db  01h


;/*--------------- Initialize global Config descriptor ----------------*/
CONFIGURATION_DESCRIPTOR:
        gCLength:                      db  gEndDescriptorLabel-gConfig_bLength
                                           ; ^^^^ Used by packet transfer routines.

        gConfig_bLength:               db 09h
        gConfig_bDescriptorType:       db CONFIG_DESCR
        gConfig_bTotalLength:          db gEndDescriptorLabel-gConfig_bLength ;
        gConfig_bCorrection:           db 0
        gConfig_bNumInterfaces:        db 1  ; NUM_OF_INTERFACES;
        gConfig_bConfigurationValue:   db 1
        gConfig_iConfiguration:        db 0
        gConfig_bmAttributes:          db 040h
        gConfig_MaxPower:              db 025h       ;50ma

;       /*------------- Initialize global Interface descriptor ---------------*/
gInterfaceDescriptor:
        gInterface_bLength:            db 09h
        gInterface_bDescriptorType:    db INTERFACE_DESCR
        gInterface_bInterfaceNumber:   db 00h
        gInterface_bAlternateSetting:  db 00h
        gInterface_bNumEndpoints:      db 02h
        gInterface_bInterfaceClass:    db 0DCh
        gInterface_bInterfaceSubClass: db 50h
        gInterface_bInterfaceProtocol: db 00h
        gInterface_iInterface:         db 0

;      /*------------- Initialize Endpoint 1 descriptor ---------------*/
gEndPoint:
        gEP_bLength:            db  07h
        gEP_bDescriptorType:    db  ENDPOINT_DESCR
        gEP_bEndpointAddress:   db  81h     ;EP address 1, in
        gEP_bmAtrributes:       db  01h     ;ISOC
        gEP_wMaxPacketSize:     dw  0800h   ;Max Packet 8 bytes
        gEP_bInterval:          db  0h

;      /*------------- Initialize Endpoint 1B descriptor ---------------*/
gEndPointB:
        gEPB_bLength:           db  07h
        gEPB_bDescriptorType:   db  ENDPOINT_DESCR
        gEPB_bEndPointAddress:  db  82h     ;EP address 2, in
        gEPB_bmAtrributes:      db  01h     ;ISOC
        gEPB_wMaxPacketSize:    dw  0800h   ;Max Packet 8 bytes
        gEPB_bInterval:         db  0h

;       /*------------- Initialize Alternate Interface descriptor 1---------------*/
gAltXface1Descriptor:
        gAltXface1_bLength:            db 09h
        gAltXface1_bDescriptorType:    db INTERFACE_DESCR
        gAltXface1_bInterfaceNumber:   db 0h
        gAltXface1_bAlternateSetting:  db 01h
        gAltXface1_bNumEndpoints:      db 2h
        gAltXface1_bInterfaceClass:    db 0DCh
        gAltXface1_bInterfaceSubClass: db 50h
        gAltXface1_bInterfaceProtocol: db 00h
        gAltXface1_iInterface:         db 0

;      /*------------- Initialize Endpoints in Alternate interface 1 ---------------*/
gEndPointAX1:
        gEPAX1_bLength:            db  07h
        gEPAX1_bDescriptorType:    db  ENDPOINT_DESCR
        gEPAX1_bEndpointAddress:   db  81h     ;EP address 1, in
        gEPAX1_bmAtrributes:       db  01h     ;ISOC
        gEPAX1_wMaxPacketSize:     dw  2000h   ;Max Packet 32 bytes
        gEPAX1_bInterval:          db  0h

gEndPointAX1B:
        gEPAX1B_bLength:           db  07h
        gEPAX1B_bDescriptorType:   db  ENDPOINT_DESCR
        gEPAX1B_bEndPointAddress:  db  81h     ;EP address 2,in
        gEPAX1B_bmAtrributes:      db  01h     ;ISOC
        gEPAX1B_wMaxPacketSize:    dw  0400h   ;Max Packet 4 bytes
        gEPAX1B_bInterval:         db  0h

;       /*------------- Initialize Alternate Interface descriptor 2---------------*/
gAltXface2Descriptor:
        gAltXface2_bLength:            db 09h
        gAltXface2_bDescriptorType:    db INTERFACE_DESCR
        gAltXface2_bInterfaceNumber:   db 00h
        gAltXface2_bAlternateSetting:  db 02h
        gAltXface2_bNumEndpoints:      db 2h
        gAltXface2_bInterfaceClass:    db 0DCh
        gAltXface2_bInterfaceSubClass: db 50h
        gAltXface2_bInterfaceProtocol: db 00h
        gAltXface2_iInterface:         db 0

;      /*------------- Initialize Endpoints in Alternate interface 2 ---------------*/
gEndPointAX2:
        gEPAX2_bLength:            db  07h
        gEPAX2_bDescriptorType:    db  ENDPOINT_DESCR
        gEPAX2_bEndpointAddress:   db  81h     ;EP address 1, in
        gEPAX2_bmAtrributes:       db  01h     ;ISOC
        gEPAX2_wMaxPacketSize:     dw  4000h   ;Max Packet 64 bytes
        gEPAX2_bInterval:          db  0h

gEndPointAX2B:
        gEPAX2B_bLength:           db  07h
        gEPAX2B_bDescriptorType:   db  ENDPOINT_DESCR
        gEPAX2B_bEndPointAddress:  db  81h     ;EP address 2,in
        gEPAX2B_bmAtrributes:      db  01h     ;ISOC
        gEPAX2B_wMaxPacketSize:    dw  0200h   ;Max Packet 2 bytes
        gEPAX2B_bInterval:         db  0h

;       /*------------- Initialize Alternate Interface descriptor 3---------------*/
gAltXface3Descriptor:
        gAltXface3_bLength:            db 09h
        gAltXface3_bDescriptorType:    db INTERFACE_DESCR
        gAltXface3_bInterfaceNumber:   db 00h
        gAltXface3_bAlternateSetting:  db 03h
        gAltXface3_bNumEndpoints:      db 2h
        gAltXface3_bInterfaceClass:    db 0DCh
        gAltXface3_bInterfaceSubClass: db 50h
        gAltXface3_bInterfaceProtocol: db 00h
        gAltXface3_iInterface:         db 0

;      /*------------- Initialize Endpoints in Alternate interface 3 ---------------*/
gEndPointAX3:
        gEPAX3_bLength:            db  07h
        gEPAX3_bDescriptorType:    db  ENDPOINT_DESCR
        gEPAX3_bEndpointAddress:   db  81h     ;EP address 1, in
        gEPAX3_bmAtrributes:       db  01h     ;ISOC
        gEPAX3_wMaxPacketSize:     dw  8000h   ;Max Packet 128 bytes
        gEPAX3_bInterval:          db  0h

gEndPointAX3B:
        gEPAX3B_bLength:           db  07h
        gEPAX3B_bDescriptorType:   db  ENDPOINT_DESCR
        gEPAX3B_bEndPointAddress:  db  81h     ;EP address 2,in
        gEPAX3B_bmAtrributes:      db  01h     ;ISOC
        gEPAX3B_wMaxPacketSize:    dw  0100h   ;Max Packet 1 bytes
        gEPAX3B_bInterval:         db  0h

;       /*------------- Initialize Alternate Interface descriptor 4---------------*/
gAltXface4Descriptor:
        gAltXface4_bLength:            db 09h
        gAltXface4_bDescriptorType:    db INTERFACE_DESCR
        gAltXface4_bInterfaceNumber:   db 00h
        gAltXface4_bAlternateSetting:  db 04h
        gAltXface4_bNumEndpoints:      db 2h
        gAltXface4_bInterfaceClass:    db 0DCh
        gAltXface4_bInterfaceSubClass: db 50h
        gAltXface4_bInterfaceProtocol: db 00h
        gAltXface4_iInterface:         db 0

;      /*------------- Initialize Endpoints in Alternate interface 4 ---------------*/
gEndPointAX4:
        gEPAX4_bLength:            db  07h
        gEPAX4_bDescriptorType:    db  ENDPOINT_DESCR
        gEPAX4_bEndpointAddress:   db  81h     ;EP address 1, in
        gEPAX4_bmAtrributes:       db  01h     ;ISOC
        gEPAX4_wMaxPacketSize:     dw  3500h   ;Max Packet 53 bytes
        gEPAX4_bInterval:          db  0h

gEndPointAX4B:
        gEPAX4B_bLength:           db  07h
        gEPAX4B_bDescriptorType:   db  ENDPOINT_DESCR
        gEPAX4B_bEndPointAddress:  db  81h     ;EP address 2,in
        gEPAX4B_bmAtrributes:      db  01h     ;ISOC
        gEPAX4B_wMaxPacketSize:    dw  0300h   ;Max Packet 3 bytes
        gEPAX4B_bInterval:         db  0h

;       /*------------- Initialize Alternate Interface descriptor 3---------------*/
gAltXface5Descriptor:
        gAltXface5_bLength:            db 09h
        gAltXface5_bDescriptorType:    db INTERFACE_DESCR
        gAltXface5_bInterfaceNumber:   db 00h
        gAltXface5_bAlternateSetting:  db 05h
        gAltXface5_bNumEndpoints:      db 2h
        gAltXface5_bInterfaceClass:    db 0DCh
        gAltXface5_bInterfaceSubClass: db 50h
        gAltXface5_bInterfaceProtocol: db 00h
        gAltXface5_iInterface:         db 0

;      /*------------- Initialize Endpoints in Alternate interface 3 ---------------*/
gEndPointAX5:
        gEPAX5_bLength:            db  07h
        gEPAX5_bDescriptorType:    db  ENDPOINT_DESCR
        gEPAX5_bEndpointAddress:   db  81h     ;EP address 1, in
        gEPAX5_bmAtrributes:       db  01h     ;ISOC
        gEPAX5_wMaxPacketSize:     dw  6700h   ;Max Packet 103 bytes
        gEPAX5_bInterval:          db  0h

gEndPointAX5B:
        gEPAX5B_bLength:           db  07h
        gEPAX5B_bDescriptorType:   db  ENDPOINT_DESCR
        gEPAX5B_bEndPointAddress:  db  81h     ;EP address 2,in
        gEPAX5B_bmAtrributes:      db  01h     ;ISOC
        gEPAX5B_wMaxPacketSize:    dw  0700h   ;Max Packet 7 bytes
        gEPAX5B_bInterval:         db  0h
gEndDescriptorLabel:


SCOPE
INIT_VARIABLES:
;Init SETUP VARIABLES
        mov R11,#0                 ; Zero out R11 to represent initial state
        mov WR28,#0                ; Access in page 00h
        mov WR30,#LOW16(SetupSeq)  ; Offset of SetupSeq var
        mov @DR28,R11              ; Initial state is wait for SETUP

                                   ;
        mov  WR10, #00h            ;
        mov  WR30, #LOW16(_EP1RxPtr);
        mov  @DR28, WR10           ;
        add  WR30, #02h            ;
        mov  @DR28, WR10           ;
        add  WR30, #02h            ;
        mov  @DR28, WR10           ;
                                   ;initialize the buffer variables for
        mov  WR30, #LOW16(_EP2RxPtr);the three endpoints
        mov  @DR28, WR10           ;
        add  WR30, #02h            ;
        mov  @DR28, WR10           ;
        add  WR30, #02h            ;
        mov  @DR28, WR10           ;
                                   ;
        mov  WR30, #LOW16(_EP3RxPtr);
        mov  @DR28, WR10           ;
        add  WR30, #02h            ;
        mov  @DR28, WR10           ;
        add  WR30, #02h            ;
        mov  @DR28, WR10           ;



        mov WR20,#0
        mov WR22, #low16(out_var_table)
        mov WR10, #0
        mov WR8,#0
?l1:    mov @DR20,R11          ;This loop zeroes out the 64 byte area
        add WR22,#1            ;at out_var_table. I'm not sure why.
        add R8,#1
        cmp R8,#64
        jne ?l1

;Zero out the output buffer area.

        mov WR20, #0
        mov WR22, #low16(OUT_BUFFERS)
        mov WR10, #0
        mov WR8,#0
?l2:    mov @DR20,R11
        add WR22,#1
        add WR8,#1
;        cmp WR8,#1000
        cmp WR8,#10h
        jne ?l2
        mov WR10,  #00h
        mov WR22,  #LOW16(DATA_PENDING)
        mov @DR20, R10
        mov WR22,  #LOW16(DIRECTION_ERROR_COUNT)
        mov @DR20, R10
        mov WR22,  #LOW16(PENDING_START_TIME)
        mov @DR20, WR10

?MoveDeviceDescriptor:
        mov     WR28, #0ffh
        mov     WR30, #LOW16(DEVICE_DESCRIPTOR)
        mov     R7, @DR28
        mov     WR26, #LOW16(gDevDescrBuffer)
        inc     R7

?DevDescrLoop:
        mov     R9, @DR28
        mov     @WR26, R9
        inc     DR28, #01h
        inc     WR26, #01h
        djnz    R7, ?DevDescrLoop

?MoveConfigDescriptor:
        mov     WR28, #0ffh
        mov     WR30, #LOW16(CONFIGURATION_DESCRIPTOR)
        mov     R7, @DR28
        mov     WR26, #LOW16(gConfigDescrBuffer)
        inc     R7

?ConfigDescrLoop:
        mov     R9, @DR28
        mov     @WR26, R9
        inc     DR28, #01h
        inc     WR26, #01h
        djnz    R7, ?ConfigDescrLoop

        RET

;#############################################################
;####### RAM MEMORY MAP ######################################
;#############################################################

;         org 00:0000h        ;0-4ff Reserved

;         org 00:0500h        ; Variables

;         org 00:0600h        ; Variables for the IN token buffers

;;
;; Data block for EP1. Contains buffer space and some management variables.
;;

define OUR_DATA_SEG, SPACE=pdata
segment OUR_DATA_SEG

org     00:2000h
_gbEP1Running:          db 0
_gbEP2Running:          db 0
_gbMPSEndPoint1:        db 0
_gbMPSEndPoint2:        db 0

org  00:D000h

EP_1_Buffer:   db [0800h] 00h
_EP1RxPtr:      dw 0
_EP1TxPtr:      dw 0
_EP1DataCount:  dw 0

;;
;; Data block for EP2. Contains buffer space and some management variables.
;;

org  00:E000h

EP_2_Buffer:   db [0800h] 00h
_EP2RxPtr:      dw 0
_EP2TxPtr:      dw 0
_EP2DataCount:  dw 0


;;
;; Data block for EP3. Contains buffer space and some management variables.
;;

org  00:F000h

EP_3_Buffer:   db [0800h] 00h
_EP3RxPtr:      dw 0
_EP3TxPtr:      dw 0
_EP3DataCount:  dw 0




org 00:9000h

SETUP_VARS:
SetupSeq:   db 0
heart_pat:  db 0
heart_bt0:  dw 0
heart_bt1:  dw 0
new_addr:   db 0

         org 00:9100h

COMMAND_BUFFER:
               db 0   ; Byte Count
StandardDeviceRequest:
bmRequestType: db 0
bRequest:      db 0
wValue:        dw 0
wIndex:        dw 0
wLength:       dw 0


 ;      define OUR_DATA_SEG, SPACE=data, ORG=00:A000h
 ;      segment OUR_DATA_SEG

      org 00:A000h


out_var_table:
out_cur_buf:   dw   0
out_cnt:       dw   0

gwControlBufferStart:   dw 0

in_var_table:

in_rd_pntr:    db   0
in_bytes_left: db   0
in_FIFO_size:  db   0
in_FIFO_empty: db   0

DATA_PENDING:  db   0
PENDING_START_TIME:  dw  0
DIRECTION_ERROR_COUNT:  db  0


org 00:B000h
gDevDescrBuffer:

org 00:B100h
gConfigDescrBuffer:

org 00:B500h
gOtherControlBuffer:

org 00:C000h
OUT_BUFFERS:

END
